/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.bmvew.bmvew.meldung;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.Stack;

import org.apache.commons.collections.map.MultiValueMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.bmvew.bmvew.BetriebsMeldungApp;
import de.bsvrz.vew.bmvew.bmvew.sender.ErinnerungsFunktion;

/**
 * Klasse die Daten einer Meldung kapselt und diese mit den Zustnden verwaltet
 * 
 * @version $Revision: 1.4 $ / $Date: 2015/06/09 13:02:12 $ / ($Author: Pittner $)
 * 
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 * 
 */
public class Meldung
{
  /**
   * Die Zustandsmaschine
   */
  private EndlicherAutomat _endlicherAutomat;

  /**
   * Die moeglichen Zustande der Meldung
   * 
   * @version $Revision: 1.4 $ / $Date: 2015/06/09 13:02:12 $ / ($Author: Pittner $)
   * 
   * @author Dambach-Werke GmbH
   * @author Timo Pittner
   * 
   */
  public static enum Zustaende {
    neu("neu"), inBearbeitung("in Bearbeitung"), Wiedervorlage("Wiedervorlage"), erledigt("erledigt"), keinenderung(
        "keine nderung");

    private String wert;

    /**
     * Konstruktor
     * 
     * @param s
     *          Initialisierung
     */
    private Zustaende(String s)
    {
      wert = s;
    }
  };

  /**
   * Der Telegrammdatensatz
   */
  private ResultData _result;

  /**
   * Das Datum des Telegrammdatensatzes
   */
  private Data _data;

  /**
   * Die Grsse des Feldes der Informationsanteile
   */
  private int MAX_SIZE_INFO_ANTEIL = 10;

  /**
   * Die Wichtigkeit
   */
  private TextValue _wichtigkeit;

  /**
   * Der Status
   */
  private TextValue _status;

  /**
   * Zeitpunkt der Wiedervorlage
   */
  private Long _wiedervorlage;

  /**
   * Loeschzeit der Meldung nach Zustandswechsel erledigt
   */
  private Long _loeschOffset;

  /**
   * Die Liste der Informationsanteile
   */
  private InformationsAnteilListe _infoAnteilListe;

  /**
   * Die Liste der Benutzeranteile
   */
  private List<BenutzerAnteil> _benutzerAnteilListe = new LinkedList<BenutzerAnteil>();

  /**
   * Der Datensatz des Urlasser
   */
  private Data _urlasser;

  /**
   * Die Meldungsgruppen
   */
  private List<String> _meldungsGruppen;

  /**
   * Der Debugger
   */
  private Debug _debug;

  /**
   * Die Pid
   */
  private String _pid;

  /**
   * Die Kennung
   */
  private String _kennung;

  /**
   * Die Id
   */
  private String _id;

  private final static Object mutex = new Object();

  /**
   * Konstruktor
   * 
   * @param result
   *          Telegrammdatensatz
   */
  public Meldung(ResultData result)
  {
    _debug = Debug.getLogger();

    _result = result;

    _data = result.getData();

    _pid = _result.getObject().getPid();

    _loeschOffset = BetriebsMeldungApp.getLoeschOffset();
    
//    synchronized (mutex)
//    {
//      int groesseInfoAnteil = 0;
//      groesseInfoAnteil = BetriebsMeldungApp.getGroesseInfoAnteil();
//      
//      if (groesseInfoAnteil > 0)
//        _infoAnteilListe = new InformationsAnteilListe(groesseInfoAnteil);
//      else
//        _infoAnteilListe = new InformationsAnteilListe(MAX_SIZE_INFO_ANTEIL);
//
//    }

    _meldungsGruppen = new ArrayList<String>();

    if (!holeConfigData())
    {
      _debug.error("Fehler KonfigData: " + result.getObject().getPid());

    }
    else
    {

      init();
      initEndlicherAutomat();

    }
  }

  public Zustand getZustand()
  {

    Zustand zustand = null;

    return zustand;
  }

  public boolean setZustand(int uebergang)
  {

    return false;
  }

  /**
   * Holt die Konfiguratiuonsdaten der Meldungen.
   * 
   * @return true, wenn die Daten erfolgreich geholt werden konnten
   */
  private boolean holeConfigData()
  {
    SystemObject so = _result.getObject();

    AttributeGroup atg = so.getDataModel().getAttributeGroup("atg.meldungEigenschaften");

    Data data = null;

    try
    {
      data = so.getConfigurationData(atg);

      _kennung = data.getItem("ApplikationsKennung").asTextValue().getValueText();

      _id = data.getItem("ID").asTextValue().getValueText();

    }
    catch (Exception e)
    {
      // TODO: handle exception
      e.getStackTrace();
    }

    if (_kennung.length() > 0 && _id != null)
      return true;

    return false;
  }

  public String getKennung()
  {
    return _kennung;
  }

  public String getId()
  {
    return _id;
  }

  /**
   * Initialisiert die Yustandsmaschine
   */
  private void initEndlicherAutomat()
  {
    Zustand neu = new Zustand("neu");
    Zustand inBearbeitung = new Zustand("in Bearbeitung");
    Zustand wiedervorlage = new Zustand("Wiedervorlage");
    Zustand erledigt = new Zustand("erledigt");

    String str = _data.getItem("Status").asTextValue().getText();

    if (str.equals(neu.toString())){
      _endlicherAutomat = new EndlicherAutomat(neu);
    }
    else if (str.equals(inBearbeitung.toString())){
      _endlicherAutomat = new EndlicherAutomat(inBearbeitung);
    }
    else if (str.equals(wiedervorlage.toString())){
      _endlicherAutomat = new EndlicherAutomat(wiedervorlage);
    }
    else if (str.equals(erledigt.toString())){
      _endlicherAutomat = new EndlicherAutomat(erledigt);
    }
    else if (str.equals("keine nderung")){
      _endlicherAutomat = new EndlicherAutomat(neu);
    }
     
    //if (!str.equals("keine nderung"))
    {

      _endlicherAutomat.addZustand(neu);
      _endlicherAutomat.addZustand(inBearbeitung);
      _endlicherAutomat.addZustand(wiedervorlage);
      _endlicherAutomat.addZustand(erledigt);

      _endlicherAutomat.addUebergang(neu, 0, inBearbeitung);
      _endlicherAutomat.addUebergang(inBearbeitung, 1, wiedervorlage);
      _endlicherAutomat.addUebergang(wiedervorlage, 2, inBearbeitung);
      _endlicherAutomat.addUebergang(inBearbeitung, 3, erledigt);

    }
  }

  /**
   * Initialisiert das Meldungsobjekt
   */
  private void init()
  {
    // TODO Auto-generated method stub

    Array a1 = _result.getData().getItem("InformationsMeldungsAnteil").asArray();

    synchronized (mutex)
    {

      int groesseInfoAnteil = BetriebsMeldungApp.getGroesseInfoAnteil();

      if (groesseInfoAnteil > 0){
        _infoAnteilListe = new InformationsAnteilListe(groesseInfoAnteil);
      }
      else{
        _infoAnteilListe = new InformationsAnteilListe(MAX_SIZE_INFO_ANTEIL);
      }

    }

    // SortedMap<Long, Data> map = new TreeMap<Long, Data>();
    MultiValueMap map = new MultiValueMap();

    Data data = null;

    for (int i = 0; i < a1.getLength(); i++)
    {
      data = a1.getItem(i);

      long l = data.getItem("Zeitstempel").asTimeValue().getMillis();

      map.put(l, data);

    }

    Set set = map.keySet();
    List list = new ArrayList();
    list.addAll(set);

    Collections.sort(list);
    
    

    for (Object key : list)
    {
      Collection coll = (Collection)map.get(key);

      for (Object object : coll)
      {

        synchronized (mutex)
        {
//          System.out.println("push : " + (Data)object);
          _infoAnteilListe.push(new InformationsAnteil((Data)object));
        }

      }
    }

    // Da die interne Liste _benutzerAnteilListe kein Stack ist,
    // wird verkehrt herum eingelesen
    Array a2 = _data.getItem("BenutzerMeldungsAnteil").asArray();

    _benutzerAnteilListe = new LinkedList<BenutzerAnteil>();

    for (int i = a2.getLength() - 1; i >= 0; i--)
    {
      Data d = a2.getItem(i);
      BenutzerAnteil ba = new BenutzerAnteil(d);

      synchronized (_benutzerAnteilListe)
      {

        _benutzerAnteilListe.add(ba);

      }

    }

    _urlasser = _result.getData().getItem("Urlasser");

  }

  public InformationsAnteilListe getInfoAnteilListe()
  {
    synchronized (mutex)
    {
      return _infoAnteilListe;
    }
  }

  public void setInfoAnteilListe(InformationsAnteilListe infoAnteilListe)
  {
    synchronized (mutex)
    {
      _infoAnteilListe = infoAnteilListe;
    }
  }

  public Data getData()
  {
    return _data;
  }

  public void setData(Data data)
  {
    _data = data;
  }

  public List<BenutzerAnteil> getBenutzerAnteilListe()
  {
    return _benutzerAnteilListe;
  }

  public void setBenutzerAnteilListe(Stack<BenutzerAnteil> benutzerAnteilListe)
  {
    _benutzerAnteilListe = benutzerAnteilListe;
  }

  public Data getUrlasser()
  {
    return _urlasser;
  }

  public void setUrlasser(Data urlasser)
  {
    _urlasser = urlasser;
  }

  public TextValue getWichtigkeit()
  {

    return _wichtigkeit;
  }

  public void setWichtigkeit(TextValue wichtigkeit)
  {

    _wichtigkeit = getData().getItem("Wichtigkeit").asTextValue();
    _wichtigkeit = wichtigkeit;
  }

  public TextValue getStatus()
  {
    return _status;
  }

  /**
   * Fhrt die Zustandsnderung durch
   * 
   * @param status
   *          Der Zustand der eingestellt werden soll
   * 
   * @return true, wenn der Zustand der Meldung erfolgreich verndert werden konnte
   */
  public boolean setStatus(TextValue status)
  {

    Data data = getData().createModifiableCopy();

    _status = data.getItem("Status").asTextValue();

    String ohneLeerzeichen = status.getText().replace(" ", "");

    Zustand zustand = null;

    SimpleDateFormat sd = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date d = new Date();

    _debug.fine("Zustandsnderung -> ist: " + _endlicherAutomat.getMomentZustand().toString() + " -> soll: "
        + status.getText());
//    System.out.println("Zustandsnderung -> ist: " + _endlicherAutomat.getMomentZustand().toString() + " -> soll: "
//        + status.getText());

    try
    {
      switch (Zustaende.valueOf(ohneLeerzeichen))
      {

        case neu:

          break;

        case inBearbeitung:

          if (_endlicherAutomat.getMomentZustand().toString().equals("neu"))
            zustand = _endlicherAutomat.uebergang(0);
          else if (_endlicherAutomat.getMomentZustand().toString().equals("Wiedervorlage"))
            zustand = _endlicherAutomat.uebergang(2);

          break;

        case Wiedervorlage:

          d.setTime(_wiedervorlage);

          if (d.getTime() > new Date().getTime() && _endlicherAutomat.getMomentZustand().toString().equals("in Bearbeitung"))
          {
            if (_endlicherAutomat.getMomentZustand().toString().equals("in Bearbeitung")){
              _debug.fine("Wiedervorlage: " + sd.format(d));
//            System.out.println("Wiedervorlage: " + sd.format(d));
              
              new ErinnerungsFunktion(this, _wiedervorlage);
              
              zustand = _endlicherAutomat.uebergang(1);              
            }else{
              _debug.fine("Zustandwechsel nicht erlaubt: " + _endlicherAutomat.getMomentZustand().toString() + " ---- > Wiedervorlage");
//              System.out.println("Zustandwechsel nicht erlaubt: " + _endlicherAutomat.getMomentZustand().toString() + " ---- > Wiedervorlage");
            }


          }
          else
            _debug.fine("Wiedervorlage: Zeitfehler");

          break;

        case erledigt:
          
          if (_endlicherAutomat.getMomentZustand().toString().equals("in Bearbeitung")){
            zustand = _endlicherAutomat.uebergang(3);         
            Long loeschzeit = System.currentTimeMillis() + _loeschOffset;
            d.setTime(loeschzeit);
            
            if (d.getTime() > new Date().getTime())
            {
              _debug.fine("Erledigt: " + sd.format(d));
              new ErinnerungsFunktion(this, loeschzeit);
            }
            else
              _debug.fine("Erledigt: Zeitfehler");
            
          }
          else{
            _debug.fine("Zustandwechsel nicht erlaubt: " + _endlicherAutomat.getMomentZustand().toString() + " ---- > erledigt");            
//            System.out.println("Zustandwechsel nicht erlaubt: " + _endlicherAutomat.getMomentZustand().toString() + " ---- > erledigt");            
          }
          
          

          break;

        case keinenderung:

          zustand = _endlicherAutomat.getMomentZustand();

          break;

        default:
          _debug.fine("kein gueltiger Zustand: " + zustand.toString());

      }
    }
    catch (Exception e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
      _debug.error("bergang ist nicht fr diesen Zustand definiert");
      return false;

    }

    // Ist ein Zustndwechsel erfolgreich, ist der zustand nicht null
    if (zustand == null)
    {
      _debug.fine("Ein Zustandswechsel konnte nicht durchgefhrt werden");
      return false;
    }

    // _status = status;
    _status.setText(zustand.toString());

    return true;
  }

  public Long getWiedervorlage()
  {
    return _wiedervorlage;
  }

  public void setWiedervorlage(Long wiedervorlage)
  {
    _wiedervorlage = wiedervorlage;
  }

  public ResultData getResult()
  {
    return _result;
  }

  public void setResult(ResultData result)
  {
    _result = result;
  }

  public EndlicherAutomat getEndlicherAutomat()
  {
    return _endlicherAutomat;
  }

  public void setEndlicherAutomat(EndlicherAutomat endlicherAutomat)
  {
    _endlicherAutomat = endlicherAutomat;
  }

  public String getPid()
  {
    return _pid;
  }

  public void setPid(String pid)
  {
    _pid = pid;
  }

  public List<String> getMeldungsGruppen()
  {
    return _meldungsGruppen;
  }

  public void setMeldungsGruppen(List<String> meldungsGruppen)
  {
    _meldungsGruppen = meldungsGruppen;
  }

}
