/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.vew.bmvew.bmvew.meldung;

import java.util.Hashtable;


/**
 * The FSM states. The default FSM State simply stores a hash table with transitions
 * along with a state name.
 */    
/**
 * Die Klasse kapselt den Zustand einer Zustandsmaschine 
 *
 * @version $Revision: 1.1 $ / $Date: 2009/09/24 12:50:45 $ / ($Author: Pittner $)
 *
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 *
 */
public class Zustand {
   
    /**
     * Der Name der Zustands
     */
    protected String _name;
    
    /**
     * Speichert die Uebergaenge 
     */
    protected Hashtable _uebergangHash = new Hashtable(); 
            
    /**
     * Konstruktor, mit Name des Zustands
     * 
     * @param name
     */
    public Zustand(String name) {
        _name = name;
    }
     
    
    public void setName(String name) {
        _name = name;
    }
   
        
    /**
     * Fgt eine Uebergang zum Zustand hinzu
     * 
     * @param uebergang
     *          Index des Uebergangs
     * @param endZustand
     *          der Endzustand
     */
    public void addUebergang(int uebergang, Zustand endZustand) {
        _uebergangHash.put(new Integer(uebergang), endZustand);            
        
    }
            
    /**
     * Realisiert den Uebergang anhand des uebergebenen Index. Wenn der uebergebene Zustand 
     * nicht definert ist, wird null zurueckgegeben
     * 
     * @param uebergang
     *          Index des Uebergangs
     * @return
     *          der Endzustand
     */
    public Zustand uebergang(int uebergang) {
        return (Zustand)_uebergangHash.get(new Integer(uebergang));
    }
    
     
    public String toString() {
        return _name;
    }

   
    
}