/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.bmvew.bmvew.regel;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.sys.funclib.debug.Debug;


/**
 * Die Klasse fhrt die definierten Aktionen aus.
 * 
 * @version $Revision: 1.3 $ / $Date: 2010/09/14 10:08:10 $ / ($Author: Pittner $)
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 */
public class Aktion
{

  /**
   * Der Debugger
   */
  private Debug _debug;

  /**
   * Der Aktionstext
   */
  private TextValue _aktion; 
  
  /**
   * Feld mit Parametertexten
   */
  private TextArray _parameter; 
  
  /**
   * Instanz des Verhalten Aktion verteilen
   */
  private AktionAusfuehrVerhalten _verteilen;

  /**
   * Instanz des Verhalten Aktion senden
   */
  private AktionAusfuehrVerhalten _senden;

  /**
   * Instanz des Verhalten Aktion verwalten
   */
  private AktionAusfuehrVerhalten _verwalten;

  /**
   * Instanz des Verhalten Aktion erweitert
   */
  private AktionAusfuehrVerhalten _erweitert;
  
  /**
   * Instanz des Verhalten Aktion erweitert
   */
  private AktionAusfuehrVerhalten _email;
  
  /**
   * Instanz des Verhalten Aktion erweitert
   */
  private AktionAusfuehrVerhalten _fax;
  
  
  /**
   * Instanz des Verhalten Aktion erweitert
   */
  private Vector<WarteThread> _verarbeiteThreads =  new Vector<WarteThread>();
  
  /**
   * Instanz des Verhalten Aktion erweitert
   */
  private Vector<WarteThread> _warteThreads =  new Vector<WarteThread>();
  
  
  private final static Object mutex = new Object();
  
  /**
   * Die Aktionen 
   *
   * @version $Revision: 1.3 $ / $Date: 2010/09/14 10:08:10 $ / ($Author: Pittner $)
   *
   * @author Dambach-Werke GmbH
   * @author Timo Pittner
   *
   */
  public static enum Aktionen {
    Verteilen("Verteilen"), Senden("Senden"), Verwalten("Verwalten"), Email("Email"), Fax("Fax"), Abbruch("Abbruch"), erweitert(
        "erweitert");

    private String wert;

    private Aktionen(String s)
    {
      wert = s;
    }
  };

  /**
   * Initialisiert die Klasse und erzeugt die spezifischen Ausfuehr-Verhalten 
   * 
   * @param aktion
   *          der Aktionstext
   * @param parameter
   *          das Feld mit Parametertexten
   *          
   * @throws IllegalArgumentException
   */
  public void initialisiere(TextValue aktion, TextArray parameter) throws IllegalArgumentException
  {

    _debug = Debug.getLogger();

    if (aktion == null || parameter == null)
    {

      throw new IllegalArgumentException();
    }

    switch (Aktionen.valueOf(aktion.getText()))
    {

      case Verteilen:
      case Senden:
      case Verwalten:
      case Email:
      case Fax:
      case Abbruch:
      case erweitert:

        break;

      default:

        throw new IllegalArgumentException();
    }

    for (int i = 0; i < parameter.getLength(); i++)
    {
      String text = parameter.getText(i);

      if (text.length() == 0)
      {

        throw new IllegalArgumentException();
      }

    }

    _aktion = aktion;
    _parameter = parameter;
    
    // Das Ausfuehr-Verhalten wird definiert 
    _verteilen = new AusfuehrenVerteilen(_parameter, _aktion);
    _senden = new AusfuehrenSenden(_parameter, _aktion);
    _verwalten = new AusfuehrenVerwalten(_parameter, _aktion);
    _erweitert = new AusfuehrenErweitert(_parameter, _aktion);
    _email = new AusfuehrenEmail(_parameter, _aktion);
    _fax = new AusfuehrenFax(_parameter, _aktion);
    
    
    Thread threadArbeiter = new Thread(new WarteThreadArbeiter(_verarbeiteThreads));
    threadArbeiter.start();
  }
  

  @Override
  public String toString()
  {
    // TODO Auto-generated method stub
    StringBuffer buffer = new StringBuffer();
    buffer.append("Parameter:");
    for (int i = 0; i < _parameter.getLength(); i++)
    {
      buffer.append("[" + _parameter.getText(i) + "]");
    }

    String s = "Aktion:" + _aktion.getText() + "; " + buffer;
    return s;
  }

  
  /**
   * Fhrt die definierten Aktionen aus
   * 
   * @param betriebsMeldung
   *          
   * @return true, wenn das Ausfuehren erfolgreich war
   */
  public boolean ausfuehren(final ResultData betriebsMeldung)
  {
    Boolean abbruch = false;
    
    switch (Aktionen.valueOf(_aktion.getText()))
    {
      
      case Verteilen:
        
        _verteilen.ausfuehrenAktion(betriebsMeldung);
        
        break;

      case Senden:

        _senden.ausfuehrenAktion(betriebsMeldung);

        break;

      case Verwalten:

        Data data = betriebsMeldung.getData();
        
        String nummer = data.getTextValue("LaufendeNummer").getText();
        String kennung = data.getTextValue("ApplikationsKennung").getText();
        String id = data.getTextValue("ID").getText();

        String pid = "meldung." + kennung + id;
        
        
        WarteThread t = new WarteThread(pid + "_" + nummer){
          @Override
          public void run()
          {
            // TODO Auto-generated method stub
            //super.run();  
            _verwalten.ausfuehrenAktion(betriebsMeldung);            
             
             
          }
        };
                
        synchronized (_verarbeiteThreads)
        {
          _verarbeiteThreads.add(t);
          _verarbeiteThreads.notify();
        }
        
        
        
        break;

      case Email:    
        _email.ausfuehrenAktion(betriebsMeldung);        
        break;

      case Fax:       
        _fax.ausfuehrenAktion(betriebsMeldung);        
        break;

      case Abbruch:        
        abbruch = true;        
        break;

      // Hier werden die Plug-Ins abgearbeitet
      case erweitert:  
        if (_erweitert.ausfuehrenAktion(betriebsMeldung))
          return true;        
        break;

      default:

    }

    if (abbruch)
      return false;

    return true;
  }
  
  class WarteThreadArbeiter implements Runnable
  {
    Vector<WarteThread> inputs;
    
    public WarteThreadArbeiter(Vector<WarteThread> inputs)
    {
      this.inputs = inputs;
    }
    
    public void run()
    {     
      while (true)
      {
        if (inputs.size() > 0)
        {
          threadsVerarbeiten();
        }
        else
        {         
          try
          {
            synchronized (inputs)
            {
              inputs.wait();             
            }
          }
          catch (InterruptedException e)
          {
            e.printStackTrace();
          }
        }
      }
    }
  }
  
  /**
   * Methode zur Verarbeitung der zu sendenden Daten.
   */
  private void threadsVerarbeiten()
  {
    while (true)
    {
      WarteThread thread = null;

      synchronized (mutex)
      {
        if (_verarbeiteThreads.size() > 0)
        {
          thread = (WarteThread)_verarbeiteThreads.remove(0);
          
          Date date = new Date();
          date.setTime(System.currentTimeMillis());
          SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
          
          
          if (_warteThreads.size() == 0)
            thread.start();
          else
            thread.start(new WarteThread[]{_warteThreads.remove(0)});
          
          _warteThreads.add(thread);
                     
        }
      }
      
      if (thread == null)
        break;
      
    }
  }
}
